@@ -176,7 +176,7 @@ paginate_update_fn <- function(
   pkg_name <- environmentName(environment(fn_call))
 
   # Ensure method can be found.
-  if (!grepl("^paws", pkg_name, perl = TRUE)) {
+  if (!startsWith(pkg_name, "paws")) {
     stopf(
       "Unknown method: `%s`. Please check service methods and try again.",
       as.character(fn)[1]
@@ -305,7 +305,7 @@ get_tokens <- function(resp, token, caller_env) {
     jmes_path <- caller_env[["jmes_path_token"]][[tkn]] %||% jmespath_index(tkn, caller_env)
     tokens[[tkn]] <- tryCatch(
       {
-        eval(parse(text = jmes_path, keep.source = FALSE), envir = environment())
+        eval(str2expression(jmes_path), envir = environment())
       },
       error = function(err) {
         # Return default character(0) for empty lists
